@@ -74,6 +74,10 @@ void store_self(pTHX, mthread* thread) {
 	SV* self = newRV_noinc(newSVuv(thread->id));
 	sv_bless(self, gv_stashpv("threads::lite::tid", TRUE));
 	hv_store(PL_modglobal, "threads::lite::self", 19, self, 0);
+
+	AV* message_cache = newAV();
+	hv_store(PL_modglobal, "threads::lite::message_cache", 28, (SV*)message_cache, 0);
+	thread->cache = message_cache;
 }
 
 mthread* S_get_self(pTHX) {
@@ -130,7 +134,7 @@ static void* run_thread(void* arg) {
 		PUSHs(ERRSV);
 	}
 	message message;
-	message_pull_stack_pushed(&message);
+	message_from_stack_pushed(&message);
 	send_listeners(thread, &message);
 	message_destroy(&message);
 
@@ -188,7 +192,7 @@ static int S_set_sigmask(sigset_t *newmask)
 
 static mthread* start_thread(mthread* thread, IV stack_size) {
 #ifdef WIN32
-	thread->handle = CreateThread(NULL, (DWORD)stack_size, run_thread, (LPVOID)thread, STACK_SIZE_PARAM_IS_A_RESERVATION, &thread->thr);
+	CreateThread(NULL, (DWORD)stack_size, run_thread, (LPVOID)thread, STACK_SIZE_PARAM_IS_A_RESERVATION, NULL);
 #else
 	int rc_stack_size = 0;
 	int rc_thread_create = 0;
@@ -215,15 +219,16 @@ static mthread* start_thread(mthread* thread, IV stack_size) {
 	}
 #  endif
 
+	pthread_t thr;
 	/* Create the thread */
 	if (! rc_stack_size) {
 #  ifdef OLD_PTHREADS_API
-		rc_thread_create = pthread_create(&thread->thr, attr, run_thread, (void *)thread);
+		rc_thread_create = pthread_create(&thr, attr, run_thread, (void *)thread);
 #  else
 #	if defined(HAS_PTHREAD_ATTR_SETSCOPE) && defined(PTHREAD_SCOPE_SYSTEM)
 		pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM);
 #	endif
-		rc_thread_create = pthread_create(&thread->thr, &attr, run_thread, (void *)thread);
+		rc_thread_create = pthread_create(&thr, &attr, run_thread, (void *)thread);
 #  endif
 	}
 	/* Now it's safe to accept signals, since we're in our own interpreter's
